from mlplatform_lib.predefinedai.pre_check_predefinedai import PreCheckPredefinedAI
from mlplatform_lib.predefinedai.predefinedai_template_test import PredefinedAITemplateTest

import pytest


@pytest.mark.usefixtures("get_server_config_path")
def test_pre_check_hyperdata(get_server_config_path):
    server_config_path = get_server_config_path
    pre_check_base = PreCheckPredefinedAI(server_config_path=server_config_path)
    pre_check_base.run()


@pytest.mark.usefixture("get_server_config_path", "get_predefinedai_image_config_path")
def test_predefinedai_template_test(get_server_config_path, get_predefinedai_image_config_path):
    server_config_path = get_server_config_path
    image_config_path = get_predefinedai_image_config_path
    predefinedai_template_test = PredefinedAITemplateTest(
        server_config_path=server_config_path, image_config_path=image_config_path
    )
    predefinedai_template_test.run()



from dataclasses import dataclass, field
from mlplatform_lib.dataclass.model.type import ModelStatus


@dataclass
class ModelInfoDto:
    id: int = field(init=False, default=0)
    type: str
    result: str
    finished_time: str = field(init=False)
    status: ModelStatus = field(init=False)



import os
from enum import Enum
import yaml


class RunMode(Enum):
    LOCAL = "local"
    KUBERNETES = "kubernetes"


class MlPlatformException(Exception):
    pass


class ApiClient:
    SECRET_LIST = ["projectId", "userId", "accessToken", "refreshToken", "authorizationType"]

    def __init__(self, server_config_path: str = None):
        # server_config_path가 none이면 pod에서 수행된다고 간주
        # 공용으로 사용하는 env 및 secret을 로드한다.
        if server_config_path is None or not os.path.isfile(server_config_path):
            print("Cannot found server_config_path. run kubernetes mode.")
            self.load_env()
            self.run_mode = RunMode.KUBERNETES
        else:
            self.load_server_config(server_config_path=server_config_path)
            self.server_config_path = server_config_path
            self.run_mode = RunMode.LOCAL

    def load_env(self):
        self.hyperdata_addr = os.environ["hyperdataAddr"]
        self.experiment_id = os.environ["experimentId"]
        self.train_id = os.environ["trainId"] if "trainId" in os.environ else None
        self.inference_id = os.environ["inferenceId"] if "inferenceId" in os.environ else None
        self.serving_id = os.environ["servingId"] if "servingId" in os.environ else None
        self.pvc_mount_path = os.environ["pvcMountPath"]
        self.serving_mount_path = os.environ["servingMountPath"] if "servingMountPath" in os.environ else None
        self.secret_mount_path = os.environ["secretMountPath"]
        file_list = [f for f in os.listdir(self.secret_mount_path)]
        for secret in self.SECRET_LIST:
            if secret not in file_list:
                print(f'Cannot found "{secret}" in secret path "{self.secret_mount_path}".')
                exit(1)

        for file in file_list:
            if file in self.SECRET_LIST:
                setattr(self, file, open(os.path.join(self.secret_mount_path, file), "r").read())
        self.Authorization = self.accessToken

    def load_server_config(self, server_config_path):
        server_config = yaml.safe_load(open(server_config_path, "r"))
        try:
            self.hyperdata_addr = server_config["hyperdataAddr"]
            self.proauth_addr = server_config["proauthAddr"]
            self.user_id = server_config["userId"]
            self.user_password = server_config["userPassword"]
            self.project_name = server_config["projectName"]
            self.experiment_id = server_config["experimentId"]
            self.train_id = server_config["trainId"]
            self.inference_id = server_config["inferenceId"]
            self.serving_id = server_config["servingId"]
            self.train_do_name = server_config["trainDoName"]
            self.inference_do_name = server_config["inferenceDoName"]
        except KeyError as e:
            print(f'Cannot found "{str(e)}" in server_config.yaml')
            exit(1)



from mlplatform_lib.predefinedai.predefinedai_api import PredefinedAIApi
from mlplatform_lib.predefinedai.predefinedai_argument_parser import PredefinedAIArgumentParser

__all__ = ["PredefinedAIApi", "PredefinedAIArgumentParser"]



from mlplatform_lib.api_client import ApiClient
from mlplatform_lib.hyperdata.hyperdata_http_client import HyperdataHttpClient, HyperdataUserAuth
from mlplatform_lib.utils.print_utils import print_header


class MlPlatformLocalChecker:
    def __init__(self, api_client: ApiClient):
        self.train_do_name = api_client.train_do_name
        self.inference_do_name = api_client.inference_do_name
        self.hyperdata_client = HyperdataHttpClient(hd_addr=api_client.hyperdata_addr)
        self.hyperdata_user_auth = HyperdataUserAuth(
            project_id=api_client.projectId, user_id=api_client.userId, authorization=api_client.Authorization
        )

        # generated
        self.train_do_id = None
        self.inference_do_id = None

    """
    def _check_mlplatform_server_connect(self):
        self._print_header(f'check mlplatform server "{self.mlplatform_addr}" connection.')
        self.mlplatform_request_header = {
            "projectId": str(self.project_id),
            "Authorization": self.access_token,
            "userId": self.user_id,
            "authorizationType": "hyperdata",
            "Content-Type": "application/json",
        }
        res = requests.get(self.mlplatform_addr, headers=self.mlplatform_request_header)
        if res.status_code != 200:
            print(
                "cannot connect to %s. mlplatform server returns status code %d"
                % (self.mlplatform_addr, res.status_code)
            )
            raise Exception(res)
        print("check mlplatform server connection end.\n")
    """

    def _check_train_do_exist(self):
        print_header(f'check is train do "{self.train_do_name}" exist.')

        do_list = self.hyperdata_client.get_do_list(self.hyperdata_user_auth)
        for do_info in do_list:
            if do_info.name == self.train_do_name:
                self.train_do_id = do_info.id
                print("check train do end.\n")
                return

        print(f'cannot found train do "{self.train_do_name}".')
        exit(1)

    def _check_inference_do_exist(self):
        print_header(f'check is inference do "{self.inference_do_name}" exist.')
        do_list = self.hyperdata_client.get_do_list(self.hyperdata_user_auth)

        for do_info in do_list:
            if do_info.name == self.inference_do_name:
                print("check inference do end.\n")
                self.inference_do_id = do_info.id
                return

        print(f'cannot found inference do "{self.inference_do_name}".')
        exit(1)

    def run(self):
        self._check_train_do_exist()
        self._check_inference_do_exist()



from dataclasses import dataclass
from typing import List, Dict
from mlplatform_lib.dataclass import DataObjectOutCol
# example
# "name": "TEST",
# "sourceTableName": "TEST",
# "subtype": "Table",
# "outCols": [
#     {
#         "name": "C1"
#     },
#     {
#         "name": "C2"
#     }
# ],
# "shareRelation": [
# ]


@dataclass
class DataObject:
    id: str
    name: str
    out_cols: List[DataObjectOutCol]
    share_relation: List[str]
    source_table_name: str
    subtype: str
    author: str
    description: str
    created_on: str
    last_edited: str



from typing import Dict, List
from dataclasses import dataclass

@dataclass
class InferencePredefinedaiDto:
    id: int
    experiment_id: int
    workflow: Dict[str, str]
    experiment_type: str
    inference_path: str
    train_id: int
    input_do_id: int
    target_do_id: int
    args: List[Dict[str, str]]
    envs: List[Dict[str, str]]


from mlplatform_lib.mlplatform.mlplatform_http_client import (
    MlPlatformHttpClient,
    MlPlatformUserAuth,
    MlPlatformRequestType,
)
from mlplatform_lib.dataclass.experiment import ExperimentPredefinedaiDto
from mlplatform_lib.dataclass.model import ModelPredefinedaiDto
from mlplatform_lib.dataclass.inference import InferencePredefinedaiDto
from typing import List
from mlplatform_lib.utils.dataclass_utils import to_dict, from_dict


class PredefinedAIHttpClient(MlPlatformHttpClient):
    def __init__(self, mlplatform_addr):
        super().__init__(mlplatform_addr=mlplatform_addr)

    def get_models(
        self, experiment_id: int, train_id: int, auth: MlPlatformUserAuth
    ) -> List[ModelPredefinedaiDto]:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            {},
            auth,
            MlPlatformRequestType.READ,
        )
        model_infos = []
        for model_info in res.data:
            model_infos.append(from_dict(ModelPredefinedaiDto, model_info))
        return model_infos

    def insert_model(
        self, experiment_id: int, train_id: int, model: ModelPredefinedaiDto, auth: MlPlatformUserAuth
    ) -> ModelPredefinedaiDto:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            to_dict(model),
            auth,
            MlPlatformRequestType.CREATE,
        )
        return from_dict(ModelPredefinedaiDto, res.data)

    def get_experiment(
        self,
        experiment_id: int,
        auth: MlPlatformUserAuth
    ) -> ExperimentPredefinedaiDto:
        result = self.send_request(
            "",
            {"experiments": experiment_id},
            {},
            {},
            auth,
            MlPlatformRequestType.READ
        )
        return from_dict(ExperimentPredefinedaiDto, result.data)

    def get_inference(
        self,
        experiment_id: int,
        inference_id: int,
        auth: MlPlatformUserAuth
    ) -> InferencePredefinedaiDto:
        result = self.send_request(
            "",
            {"experiments": experiment_id, "inferences": inference_id},
            {},
            {},
            auth,
            MlPlatformRequestType.READ
        )
        return from_dict(InferencePredefinedaiDto, result.data)

    def update_inference(
        self,
        experiment_id: int,
        inference_predefinedai_dto: InferencePredefinedaiDto,
        auth: MlPlatformUserAuth
    ) -> InferencePredefinedaiDto:
        result = self.send_request(
            "inferences",
            {"experiments": experiment_id},
            {},
            to_dict(inference_predefinedai_dto),
            auth,
            MlPlatformRequestType.UPDATE
        )
        return from_dict(InferencePredefinedaiDto, result.data)


from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.dataclass.experiment.type import ExperimentType
from mlplatform_lib.dataclass.model.type import ModelStatus
from mlplatform_lib.dataclass.model import ModelPredefinedaiDto, ModelInfoDto
from mlplatform_lib.dataclass import InsertTupleObject, experiment
from mlplatform_lib.mlplatform.mlplatform_http_client import MlPlatformUserAuth
from mlplatform_lib.predefinedai.predefinedai_http_client import PredefinedAIHttpClient
import os
from typing import List, Optional
from mlplatform_lib.hyperdata import HyperdataApi
import pandas as pd


class PredefinedAIApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        if api_client.run_mode == RunMode.KUBERNETES:
            self.predefinedai_client = PredefinedAIHttpClient(mlplatform_addr=os.environ["mlplatformAddr"])
            self.hyperdata_api = HyperdataApi(api_client=self.api_client)

            self.mlplatform_auth = MlPlatformUserAuth(
                project_id=self.api_client.projectId,
                user_id=self.api_client.userId,
                authorization=self.api_client.Authorization,
                authorization_type=self.api_client.authorizationType,
            )

        self.experiment_id = self.api_client.experiment_id
        self.train_id = self.api_client.train_id
        self.inference_id = self.api_client.inference_id
        self.target_do_id = os.environ["targetDoId"] if "targetDoId" in os.environ else None

    def get_models(self) -> Optional[List[ModelPredefinedaiDto]]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.predefinedai_client.get_model_infos(
                experiment_id=self.api_client.experiment_id,
                train_id=self.api_client.train_id,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip get_model_infos.")
            return None

    def insert_model(self, model: ModelPredefinedaiDto) -> Optional[ModelPredefinedaiDto]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.PREDEFINEDAI
            return self.predefinedai_client.insert_model(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model=model,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip insert_model_info.")
            model.id = 1
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.PREDEFINEDAI
            return model

    def insert_model_info(self, model_id: int, model_info: ModelInfoDto) -> ModelInfoDto:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model_info.status = ModelStatus.SUCCESS
            return self.predefinedai_client.insert_model_info(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model_id=model_id,
                model_info=model_info,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip insert_visualizations.")

    def _get_inference_result_table_name(self) -> str:
        pdai_experiment_dto = self.predefinedai_client.get_experiment(
            experiment_id=self.experiment_id,
            auth=self.mlplatform_auth
        )

        return f"PREDEFINEDAI_{pdai_experiment_dto.name}_{str(pdai_experiment_dto.id)}"

    def upload_inference_csv(self, inference_csv_path: str):
        if self.api_client.run_mode == RunMode.KUBERNETES:
            # 1. check is target do exists
            inference_data = pd.read_csv(inference_csv_path)
            
            table_name = self._get_inference_result_table_name()
            self.target_do_id = self.hyperdata_api.create_inference_result(
                table_name=table_name,
                object_name=table_name,
                columns=inference_data.columns.tolist()
            )

            insert_tuple_object = InsertTupleObject(
                isTruncated="True",
                targetColNames=inference_data.columns.tolist(),
                tableData=inference_data.values.tolist()
            )

            self.hyperdata_api.hyperdata_client.insert_dataobject_tuple(
                auth=self.hyperdata_api.hyperdata_auth,
                dataobject_id=self.target_do_id,
                insert_tuple_objects=insert_tuple_object
            )

            pdai_inference_dto = self.predefinedai_client.get_inference(
                experiment_id=self.experiment_id,
                inference_id=self.inference_id,
                auth=self.mlplatform_auth
            )
            pdai_inference_dto.target_do_id = self.target_do_id
            self.predefinedai_client.update_inference(
                experiment_id=self.experiment_id,
                inference_predefinedai_dto=pdai_inference_dto,
                auth=self.mlplatform_auth
            )

            os.remove(inference_csv_path)
        else:
            print("Current mode is local, Skip upload_inference_csv.")



from dataclasses import dataclass
from .column_of_table_from_default_source import ColumnOfTableFromDefaultSource
from typing import List


@dataclass
class TableFromDefaultDataSourceInfo:
    table_name: str
    column_list: List[ColumnOfTableFromDefaultSource]



import json
from mlplatform_lib.api_client import ApiClient
from mlplatform_lib.hyperdata.hyperdata_http_client import HyperdataHttpClient
import requests
from mlplatform_lib.utils.print_utils import print_header


class HyperdataLocalChecker:
    def __init__(self, api_client: ApiClient):
        self.hyperdata_addr = api_client.hyperdata_addr
        self.proauth_addr = api_client.proauth_addr
        self.user_id = api_client.user_id
        self.user_password = api_client.user_password
        self.project_name = api_client.project_name
        self.hyperdata_client = HyperdataHttpClient(hd_addr=self.hyperdata_addr)

        # generated
        self.hyperdata_client = None
        self.access_token = None
        self.hyperdata_token_header = None
        self.project_id = None

    def _check_hyperdata_proauth_server_connect(self):
        print_header(f'check proauth server "{self.proauth_addr}" connection.')
        res = requests.post(
            url=self.proauth_addr + "/proauth/oauth/authenticate",
            headers={"Content-Type": "application/json;charset=UTF-8"},
            data=json.dumps({"user_id": self.user_id, "password": self.user_password}),
        )
        if res.status_code != 200:
            print(
                "cannot connect to %s. proauth server returns status code %d"
                % (self.proauth_addr, res.status_code)
            )
            raise Exception(res)

        response = json.loads(res.text)
        if response["result"] == "false":
            raise Exception(response["error"])
        self.access_token = response["token"]
        self.hyperdata_token_header = {"Authorization": self.access_token}
        print("check proauth server connection end.\n")

    def _check_hyperdata_project(self):
        print_header(f'check is hyperdata project "{self.project_name}" exist.')

        project_get_url = self.hyperdata_addr + "/hyperdata/web-service/projects"
        res = requests.get(project_get_url, headers=self.hyperdata_token_header)
        if res.status_code != 200:
            print(
                "cannot connect to %s. hyperdata server returns status code %d"
                % (self.hyperdata_addr, res.status_code)
            )
            raise Exception(res)

        project_infos = json.loads(res.text)["dto"]["project"]
        project_id = None
        for project_info in project_infos:
            if project_info["name"] == self.project_name:
                project_id = project_info["id"]
                break
        if project_id is None:
            print(f"Cannot found project {self.project_name}. Please check hyperdata.")
            raise Exception("_check_hyperdata_project failed.")
        else:
            self.project_id = project_id

        print("check hyperdata server connection end.\n")

    def run(self):
        self._check_hyperdata_proauth_server_connect()
        self._check_hyperdata_project()



from typing import List
from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.dataclass import (
    ColumnOfTableFromDefaultSource,
    TableFromDefaultDataSourceInfo, DataObject,
    DataObjectOutCol,
    DataObjectInfoList
)
from mlplatform_lib.hyperdata.hyperdata_http_client import HyperdataHttpClient, HyperdataUserAuth, HYPERDATA_DATETIME_FORMAT
from mlplatform_lib.hyperdata.hyperdata_local_checker import HyperdataLocalChecker
import os
import pathlib
import pandas as pd
from datetime import datetime


class HyperdataApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        if self.api_client.run_mode == RunMode.LOCAL:
            self.local_checker = HyperdataLocalChecker(api_client=api_client)
            self.local_checker.run()
            self.api_client.projectId = self.local_checker.project_id
            self.api_client.userId = self.api_client.user_id
            self.api_client.Authorization = self.local_checker.access_token

        self.hyperdata_client = HyperdataHttpClient(hd_addr=self.api_client.hyperdata_addr)
        self.hyperdata_auth = HyperdataUserAuth(
            project_id=self.api_client.projectId,
            user_id=self.api_client.userId,
            authorization=self.api_client.Authorization,
        )

    def download_csv(self, do_id: int, data_rootpath: str) -> str:
        result, sep, line_delim = self.hyperdata_client.download_do_to_csv(self.hyperdata_auth, do_id)

        pathlib.Path(data_rootpath).mkdir(parents=True, exist_ok=True)
        csv_path = os.path.join(data_rootpath, "%d.csv" % int(do_id))

        csv_str = result.data.decode("utf-8")
        if line_delim != "\n":
            csv_str.replace(line_delim, "\n")
        with open(csv_path, "w") as f:
            f.write(csv_str)

        if sep != ",":
            data = pd.read_csv(csv_path, sep=sep)
            data.to_csv(csv_path, sep=",", index=False)

        return csv_path

    def create_inference_result(self, table_name: str, object_name: str, columns: List[str]) -> int:
        default_datasource_id = self.hyperdata_client.get_default_datasource_id(auth=self.hyperdata_auth)
        table_list = self.hyperdata_client.get_dataobject_tables(
            auth=self.hyperdata_auth,
            datasource_id=default_datasource_id
        )

        if table_name not in [table_info.name for table_info in table_list.db_source_data_object_candidate]:
            self.hyperdata_client.create_dataobject_table(
                auth=self.hyperdata_auth,
                datasource_id=default_datasource_id,
                table_from_default_data_source=TableFromDefaultDataSourceInfo(
                    table_name=table_name,
                    column_list=[
                        ColumnOfTableFromDefaultSource(
                            column_name=column,
                            type="VARCHAR"
                        ) for column in columns
                    ]
                )
            )
        else:
            print(f"table {table_name} already exists. skip create dataobject table")

        do_info_list = self.hyperdata_client.get_do_list_by_datasource_id(
            auth=self.hyperdata_auth,
            datasource_id=self.hyperdata_client.get_default_datasource_id(
                auth=self.hyperdata_auth
            )
        )

        if object_name not in [object_info.name for object_info in do_info_list.data_object_info_list]:
            do_info_list = self.hyperdata_client.create_dataobjects(
                auth=self.hyperdata_auth,
                datasource_id=default_datasource_id,
                dataobjects=DataObjectInfoList(
                    data_object_info_list=[
                        DataObject(
                            id=0,
                            name=object_name,
                            source_table_name=table_name,
                            subtype="Inference Result",
                            out_cols=[
                                DataObjectOutCol(
                                    name=column
                                ) for column in columns
                            ],
                            share_relation=[],
                            author="",
                            description="",
                            created_on=datetime.now().strftime(HYPERDATA_DATETIME_FORMAT),
                            last_edited=datetime.now().strftime(HYPERDATA_DATETIME_FORMAT)
                        )
                    ]
                )
            )
            return int(do_info_list.data_object_info_list[0].id)
        else:
            for object_info in do_info_list.data_object_info_list:
                if object_info.name == object_name:
                    return int(object_info.id)
            print(f"dataobject {object_name} already exists. skip create dataobject")




from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.mlplatform.mlplatform_http_client import MlPlatformUserAuth
from mlplatform_lib.mllab.mllab_http_client import MllabHttpClient
from mlplatform_lib.hyperdata.hyperdata_http_client import HyperdataHttpClient, HyperdataUserAuth
from mlplatform_lib.hyperdata.hyperdata_api import HyperdataApi
from mlplatform_lib.dataclass.model.model_dto import ModelDto
from mlplatform_lib.dataclass import (
    InsertTupleObject,
    TableColumnInfo,
)
import os
import pandas as pd
import sys
from typing import List, Optional


class MllabImageType:
    TENSORFLOW_V1 = "tf_v1.15.2"
    TENSORFLOW_V2 = "tf_v2.1.0"
    TORCH = "torch_v1.6.0"


if os.environ["image"] == MllabImageType.TENSORFLOW_V1 or os.environ["image"] == MllabImageType.TENSORFLOW_V2:
    import tensorflow as tf
elif os.environ["image"] == MllabImageType.TORCH:
    import torch


class MllabApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        if api_client.run_mode == RunMode.KUBERNETES:
            self.mllab_mlplatform_client = MllabHttpClient(mlplatform_addr=os.environ["mlplatformAddr"])
            self.mllab_hyperdata_client = HyperdataHttpClient(self.api_client.hyperdata_addr)
            self.mllab_hyperdata_api = HyperdataApi(api_client=self.api_client)
            self.mlplatform_auth = MlPlatformUserAuth(
                project_id=self.api_client.projectId,
                user_id=self.api_client.userId,
                authorization=self.api_client.Authorization,
                authorization_type=self.api_client.authorizationType,
            )
            self.hyperdata_auth = HyperdataUserAuth(
                project_id=self.api_client.projectId,
                user_id=self.api_client.userId,
                authorization=self.api_client.Authorization,
            )
        self.experiment_id = self.api_client.experiment_id
        self.image = os.environ["image"]

    def get_model_list(self) -> Optional[List[ModelDto]]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.mllab_mlplatform_client.get_model_list(
                experiment_id=self.experiment_id, auth=self.mlplatform_auth
            )
        else:
            print("Current mode is local, Skip get_list_model.")
            return None

    def get_model(self, model_path: str = None):
        if self.api_client.run_mode == RunMode.KUBERNETES:
            if model_path is None:
                latest_model_dto = self.mllab_mlplatform_client.get_latest_model(
                    experiment_id=self.experiment_id, auth=self.mlplatform_auth
                )
                model_path = latest_model_dto.model_path

            if self.image == MllabImageType.TENSORFLOW_V1:
                return tf.compat.v2.saved_model.load(model_path + "/")
            elif self.image == MllabImageType.TENSORFLOW_V2:
                return tf.saved_model.load(model_path + "/")
            elif self.image == MllabImageType.TORCH:
                sys.path.append(model_path + "/")
                from network_class import Net

                net = Net()
                return net, torch.load(model_path + "/" + "model.pt")
        else:
            print("Current mode is local, Skip get_list_do.")
            return None

    def save_model(self, model, signatures=None) -> bool:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            export_dir = self.api_client.pvc_mount_path + "/model/model-" + self.api_client.train_id + "/1"
            if not os.path.isdir(export_dir):
                os.makedirs(export_dir)

            if self.image == MllabImageType.TENSORFLOW_V1:
                tf.saved_model.save(model, export_dir, signatures)
            elif self.image == MllabImageType.TENSORFLOW_V2:
                tf.saved_model.save(model, export_dir, signatures)
            elif self.image == MllabImageType.TORCH:
                torch.save(model.state_dict(), export_dir + "/model.pt", _use_new_zipfile_serialization=False)
                if os.path.isfile(self.api_client.pvc_mount_path + "/user_files/user_train.py"):
                    fin = open(self.api_client.pvc_mount_path + "/user_files/user_train.py", "r")
                    fout = open(export_dir + "/network_class.py", "w+")
                    for line in fin:
                        if line.strip().startswith("import pandas as pd"):
                            continue
                        elif line.strip().startswith("import os"):
                            continue
                        elif line.strip().startswith("from sklearn.model_selection import train_test_split"):
                            continue
                        elif line.strip().startswith(
                            "from sklearn.metrics import precision_recall_fscore_support"
                        ):
                            continue
                        elif line.strip().startswith("import hyper"):
                            continue
                        else:
                            fout.write(line)

            model_dto = self.mllab_mlplatform_client.get_model_by_id(
                experiment_id=self.experiment_id, train_id=self.api_client.train_id, auth=self.mlplatform_auth
            )
            model_dto.model_path = export_dir
            return self.mllab_mlplatform_client.update_model(
                experiment_id=self.experiment_id,
                train_id=self.api_client.train_id,
                dto=model_dto,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip get_list_do.")
            return None

    def get_do_list(self) -> dict:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.mllab_hyperdata_client.get_do_simple_list(auth=self.hyperdata_auth)
        else:
            print("Current mode is local, Skip get_list_do.")
            return None

    def get_do(self, do_id: str, char_encoding="euc-kr") -> pd.DataFrame:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            file_name = do_id + ".csv"
            dir_name = self.api_client.pvc_mount_path + "/data"
            full_path = dir_name + "/" + file_name
            if os.path.exists(full_path):
                result = pd.read_csv(full_path, encoding=char_encoding)
                return result
           
            do_path = self.mllab_hyperdata_api.download_csv(
                do_id=do_id,
                data_rootpath=dir_name
            )

            if os.path.exists(do_path) :
                result = pd.read_csv(do_path, encoding=char_encoding)
            else :
                print("Fail")
                result = None
            return result
        else:
            print("Current mode is local, Skip get_do.")
            return None

    def get_do_meta(self, do_id: str) -> List[TableColumnInfo]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            tableDescInfo = self.mllab_hyperdata_client.get_do_detail_info(
                do_id=do_id, auth=self.hyperdata_auth
            )
            return tableDescInfo.col_info_list
        else:
            print("Current mode is local, Skip get_do_meta.")
            return None

    def save_do(self, input_data: pd.DataFrame, do_id: str, is_truncated: str = False) -> bool:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            print(self.api_client.inference_id)
            if self.api_client.inference_id is not None:
                file_dir = (
                    self.api_client.pvc_mount_path + "/inference/inference-" + self.api_client.inference_id
                )
                file_path = file_dir + "/inference-result.csv"
                if not os.path.isdir(file_dir):
                    os.mkdir(file_dir)
                input_data.to_csv(file_path, index=False, header=True)
            else:
                print("inference id is None")
                return False

            inference_dto = self.mllab_mlplatform_client.get_inference_by_id(
                experiment_id=self.experiment_id,
                inference_id=self.api_client.inference_id,
                auth=self.mlplatform_auth,
            )
            inference_dto.inference_path = file_path
            self.mllab_mlplatform_client.update_inference(
                experiment_id=self.experiment_id, dto=inference_dto, auth=self.mlplatform_auth,
            )

            insert_tuple_object = InsertTupleObject(
                isTruncated=is_truncated,
                targetColNames=input_data.columns.tolist(),
                tableData=input_data.values.tolist(),
            )
            res = self.mllab_hyperdata_client.insert_dataobject_tuple(
                auth=self.hyperdata_auth, dataobject_id=do_id, insert_tuple_objects=insert_tuple_object
            )
            if res.status_code == 200:
                return True
            else:
                return False



import os
from typing import List, Optional

from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.automl.automl_http_client import AutomlHttpClient
from mlplatform_lib.dataclass.experiment.type import ExperimentType
from mlplatform_lib.dataclass.model import ModelInfoDto, ModelAutomlDto, ModelAutomlBestDto
from mlplatform_lib.dataclass.model.type import ModelStatus
from mlplatform_lib.mlplatform.mlplatform_http_client import MlPlatformUserAuth


class AutomlApi:
    def __init__(self, api_client: ApiClient = None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        if api_client.run_mode == RunMode.KUBERNETES:
            self.automl_client = AutomlHttpClient(mlplatform_addr=os.environ["mlplatformAddr"])
            self.mlplatform_auth = MlPlatformUserAuth(
                project_id=self.api_client.projectId,
                user_id=self.api_client.userId,
                authorization=self.api_client.Authorization,
                authorization_type=self.api_client.authorizationType,
            )
        self.experiment_id = self.api_client.experiment_id
        self.train_id = self.api_client.train_id
        self.inference_id = self.api_client.inference_id

    def get_models(self) -> Optional[List[ModelAutomlDto]]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.automl_client.get_models(
                experiment_id=self.experiment_id, train_id=self.train_id, auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip get_models.")
            return None

    def insert_model(self, model: ModelAutomlDto) -> Optional[ModelAutomlDto]:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.AUTOML
            return self.automl_client.insert_model(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model=model,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip insert_model.")
            model.id = 1
            model.train_id = self.train_id
            model.status = ModelStatus.SUCCESS
            model.experiment_type = ExperimentType.AUTOML
            return model

    def insert_model_info(self, model_id: int, model_info: ModelInfoDto) -> ModelInfoDto:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            model_info.status = ModelStatus.SUCCESS
            return self.automl_client.insert_model_info(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model_id=model_id,
                model_info=model_info,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip insert_visualizations.")

    def upload_inference_csv(self, inference_csv_path: str):
        if self.api_client == RunMode.KUBERNETES:
            self.automl_client.upload_inference_csv(
                experiment_id=self.experiment_id,
                inference_id=self.inference_id,
                inference_csv_path=inference_csv_path,
                auth=self.mlplatform_auth,
            )
            os.remove(inference_csv_path)
        else:
            print("Current mode is local, Skip upload_inference_csv.")

    def insert_model_automl_best(self, model_automl_best: ModelAutomlBestDto) -> ModelAutomlBestDto:
        if self.api_client.run_mode == RunMode.KUBERNETES:
            return self.automl_client.insert_model_automl_best(
                experiment_id=self.experiment_id,
                train_id=self.train_id,
                model_automl_best=model_automl_best,
                auth=self.mlplatform_auth,
            )
        else:
            print("Current mode is local, Skip insert_model_automl_best.")



from mlplatform_lib.dataclass import (
    DataObject,
    TableDescInfo,
    InsertTupleObject,
    TableFromDefaultDataSourceInfo,
    DataObjectInfoList,
    DbSourceDataObjectCandidate,
    DbSourceDataObjectCandidateList
)
from dataclasses import dataclass
from enum import Enum
import json
import requests
import os
import time
from typing import Optional, Dict, List, Union, Tuple
from mlplatform_lib.utils.dataclass_utils import from_dict, to_dict


HYPERDATA_DATETIME_FORMAT = "%Y.%m.%d %H.%M.%S"
HYPERDATA_DATETIME_INIT_STR = "1970.01.01 00.00.00"


class HyperdataRequestException(Exception):
    status_code = 200

    def __init__(self, message, status_code=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code


class HyperdataRequestType(Enum):
    CREATE = 0
    READ = 1
    UPDATE = 2
    DELETE = 3
    DOWNLOAD = 4
    READ_WITH_FORM_DATA = 5


@dataclass
class HyperdataUserAuth:
    project_id: str
    user_id: str
    authorization: str


@dataclass
class HyperdataRequestResult:
    data: Optional[Union[Dict, List[Dict], bytes]]
    content: Optional[str]
    status_code: Optional[int]


class HyperdataHttpClient:
    def __init__(self, hd_addr):
        self.base_url = hd_addr + "/hyperdata/web-service"

    def send_request(
        self,
        service: str,
        rest: Optional[dict],
        query: Optional[dict],
        data: Optional[dict],
        auth: HyperdataUserAuth,
        request_type: HyperdataRequestType,
    ) -> HyperdataRequestResult:
        headers = {
            "Content-Type": "application/json",
            "Authorization": auth.authorization,
            "userId": auth.user_id,
        }

        if rest is None:
            rest = {}

        if query is None:
            query = {}

        if data is None:
            data = {}

        if service == "dataObjectTuple":
            headers["doType"] = "inferenceResult"
            
        if request_type == HyperdataRequestType.DOWNLOAD:
            headers["ProObjectWebFileTransfer"] = "true"
            query["action"] = "Download"

        url = self.base_url + "/projects/" + str(auth.project_id)
        for key, val in rest.items():
            url = url + "/" + str(key) + "/" + str(val)
        if service != "" and service is not None:
            url = url + "/" + service

        if len(query) != 0:
            url += "?"
            for idx, (key, val) in enumerate(query.items()):
                if idx != 0:
                    url += "&"
                url = url + str(key) + "=" + str(val)

        if data != "" and data is not None:
            data = {"dto": data}
        else:
            data = {"dto": ""}

        if request_type == HyperdataRequestType.READ or request_type == HyperdataRequestType.DOWNLOAD:
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.CREATE:
            response = requests.post(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.UPDATE:
            response = requests.put(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.DELETE:
            response = requests.delete(url, headers=headers, data=json.dumps(data), verify=False)
        elif request_type == HyperdataRequestType.READ_WITH_FORM_DATA:
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=False)
        else:
            return HyperdataRequestResult(None, None, None)

        if response.status_code == 200:
            if request_type == HyperdataRequestType.DOWNLOAD:
                return HyperdataRequestResult(response.content, None, response.status_code)
            else:
                return HyperdataRequestResult(
                    response.json(), response.content.decode(), response.status_code
                )
        else:
            print(response.content.decode())
            print(
                (
                    f"cannot connect to {self.base_url}. "
                    f"hyperdata server returns status code {response.status_code}"
                )
            )
            raise HyperdataRequestException(response.content.decode(), response.status_code)

    # TODO: hyperdata datasource에 do 정보들을 다가지고 오는 so 구현 필요
    def get_do_list(self, auth: HyperdataUserAuth) -> List[DataObject]:
        result = self.send_request(
            "automlExperimentDataObjectList", {}, {}, {}, auth, HyperdataRequestType.READ
        )
        result_data = result.data["dto"]["dataObjectInfoList"]

        do_list = []
        for do_info in result_data:
            do_list.append(self.get_do_info(auth, do_info["id"]))
        return do_list

    def get_do_simple_list(self, auth: HyperdataUserAuth) -> dict:
        result = self.send_request(
            "automlExperimentDataObjectList", {}, {}, {}, auth, HyperdataRequestType.READ
        )
        result_data = result.data["dto"]["dataObjectInfoList"]
        do_list = {}
        for do_info in result_data:
            do_list[do_info["name"]] = do_info["id"]
        return do_list

    def get_do_list_by_datasource_id(self, auth: HyperdataUserAuth, datasource_id: int) -> DataObjectInfoList:
        result = self.send_request(
            "dataobjects",
            {"datasources": datasource_id},
            {},
            {},
            auth,
            HyperdataRequestType.READ
        )
        result_data = result.data["dto"]["dataObjectInfoList"]
        return DataObjectInfoList(
            data_object_info_list=[
                from_dict(DataObject, do_info) for do_info in result_data
            ]
        )


    def get_do_info(self, auth: HyperdataUserAuth, do_id: int) -> DataObject:
        result = self.send_request(
            "", {"dataobjects": do_id}, {"action": "Detail"}, {}, auth, HyperdataRequestType.READ,
        )
        return from_dict(DataObject, result.data["dto"])

    #    def get_do_samples(self, auth: HyperdataUserAuth, dataobject_id: int, sample_size: int):
    #        return self.send_request(
    #            "",
    #            {"dataobjects": dataobject_id},
    #            {"action": "Sample", "size": sample_size},
    #            {},
    #            auth,
    #            HyperdataRequestType.READ,
    #        )

    # 아직 공식 hyperdata so에서 값이 이상하게 들어와 직접 만든걸 사용
    def get_do_samples(
        self, auth: HyperdataUserAuth, dataobject_id: int, sample_size: int
    ) -> HyperdataRequestResult:
        result = self.send_request(
            "automlExperimentDataObjectSample",
            {"dataobjects": dataobject_id},
            {"size": sample_size},
            {},
            auth,
            HyperdataRequestType.READ,
        )
        fixed_table_string = []
        for row in json.loads(result.data["dto"]["tableString"]):
            fixed_row = {}
            for k, v in row.items():
                fixed_row[k[1:-1]] = v
            fixed_table_string.append(fixed_row)
        result.data["dto"]["tableString"] = json.dumps(fixed_table_string)
        return result

    def get_do_detail_info(self, auth: HyperdataUserAuth, do_id: int):
        result = self.send_request(
            "",
            {"isedit": 0, "subtype": "Table", "dataobjects": do_id},
            {"action": "Desc"},
            {},
            auth,
            HyperdataRequestType.READ,
        )
        return from_dict(TableDescInfo, result.data["dto"])

    def download_do_to_csv(
        self, auth: HyperdataUserAuth, do_id: int
    ) -> Tuple[HyperdataRequestResult, str, str]:
        result = self.send_request(
            "", {"dataobjects": do_id}, {"action": "Detail"}, {}, auth, HyperdataRequestType.READ,
        )

        table_name = result.data["dto"]["sourceTableName"]
        subtype = result.data["dto"]["subtype"]
        datasource_id = result.data["dto"]["datasourceId"]

        def _download_do_to_csv():
            result = self.send_request(
                "",
                {"isedit": 0, "subtype": subtype, "dataobjects": do_id},
                {"action": "Desc"},
                {},
                auth,
                HyperdataRequestType.READ,
            )
            cols = result.data["dto"]["colInfoList"]
            col_select_opts = []
            for col in cols:
                col_select_opts.append(
                    {"type": col["type"], "alias": col["alias"], "name": col["alias"], "userExpr": ""}
                )
            return self.send_request(
                "",
                {"dataobjects": do_id},
                {},
                {
                    "id": "task1",
                    "name": "1",
                    "itemId": "selector",
                    "inputTables": [{"name": table_name, "cols": cols}],
                    "selectorProperty": {"columnSelectOptions": col_select_opts, "rowSelectOptions": []},
                    "outputTable": {"name": table_name},
                },
                auth,
                HyperdataRequestType.DOWNLOAD,
            )

        result = self.send_request(
            "", {"datasources": datasource_id}, {"action": "Detail"}, {}, auth, HyperdataRequestType.READ,
        )
        do_type_check = None
        try:
            do_type_check = (
                "fileSourceParam" in result.data["dto"]
                and "fileType" in result.data["dto"]["fileSourceParam"]
                and result.data["dto"]["fileSourceParam"]["fileType"] is not None
                and result.data["dto"]["fileSourceParam"]["fileType"].lower().find("sam") != -1
            )
        except Exception:
            return _download_do_to_csv(), ",", "\\n"

        if do_type_check:
            return (
                self.send_request(
                    "automlDataObjectCSV",
                    {"dataobjects": do_id},
                    {"action": "Download"},
                    {},
                    auth,
                    HyperdataRequestType.DOWNLOAD,
                ),
                result.data["dto"]["fileSourceParam"]["fieldDelimiter"],
                result.data["dto"]["fileSourceParam"]["lineDelimiter"],
            )
        else:
            return _download_do_to_csv(), ",", "\\n"

    def get_default_datasource_id(self, auth: HyperdataUserAuth) -> int:
        result = self.send_request("dataSourceDefault", {}, {}, {}, auth, HyperdataRequestType.READ)
        return int(result.data["dto"]["defaultDataSourceId"])

    def get_dataobject_tables(
        self,
        auth: HyperdataUserAuth,
        datasource_id: int
    ):
        result = self.send_request(
            "",
            {"datasources": datasource_id},
            {"action": "Desc"},
            {},
            auth,
            HyperdataRequestType.READ
        )

        result_data = result.data["dto"]["DbSourceDataObjectCandidate"]
        return DbSourceDataObjectCandidateList(
            db_source_data_object_candidate=[
                from_dict(DbSourceDataObjectCandidate, table_info) for table_info in result_data
            ]
        )


    def create_dataobject_table(
        self,
        auth: HyperdataUserAuth,
        datasource_id: int,
        table_from_default_data_source: TableFromDefaultDataSourceInfo,
    ) -> None:
        self.send_request(
            "dataobjects",
            {"datasources": datasource_id},
            {"action": "CreateTable"},
            to_dict(table_from_default_data_source),
            auth,
            HyperdataRequestType.CREATE,
        )

    def create_dataobjects(
        self, auth: HyperdataUserAuth, datasource_id: int, dataobjects: DataObjectInfoList
    ) -> DataObjectInfoList:
        result = self.send_request(
            "dataobjects",
            {"datasources": datasource_id},
            {},
            to_dict(dataobjects),
            auth,
            HyperdataRequestType.CREATE,
        )
        result_data = result.data["dto"]["dataObjectInfoList"]
        return DataObjectInfoList(
            data_object_info_list=[
                from_dict(DataObject, do_info) for do_info in result_data
            ]
        )


    def insert_dataobject_tuple(
        self, auth: HyperdataUserAuth, dataobject_id: int, insert_tuple_objects: InsertTupleObject,
    ) -> HyperdataRequestResult:
        return self.send_request(
            "dataObjectTuple",
            {"dataobjects": dataobject_id},
            {},
            {"jsonString": json.dumps(insert_tuple_objects.__dict__)},
            auth,
            HyperdataRequestType.UPDATE,
        )

    def delete_dataobjects(self, auth: HyperdataUserAuth, dataobject_id: int) -> HyperdataRequestResult:
        return self.send_request(
            "datasources",
            {},
            {},
            {"dataobjectId": [dataobject_id], "datasourceId": [], "fileType": []},
            auth,
            HyperdataRequestType.DELETE,
        )



from mlplatform_lib.api_client import ApiClient, RunMode
from mlplatform_lib.dataclass.model import ModelInfoDto
from mlplatform_lib.hyperdata.hyperdata_api import HyperdataApi
from mlplatform_lib.mlplatform.mlplatform_http_client import MlPlatformHttpClient, MlPlatformUserAuth
from mlplatform_lib.mlplatform.mlplatform_local_checker import MlPlatformLocalChecker
import os
from typing import List
from pathlib import Path


class MlPlatformApi:
    def __init__(self, api_client: ApiClient = None, local_mount_path="./tmp"):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.hyperdata_api = HyperdataApi(api_client=self.api_client)
        self.experiment_id = self.api_client.experiment_id
        self.train_id = self.api_client.train_id
        self.inference_id = self.api_client.inference_id
        if self.api_client.run_mode == RunMode.LOCAL:
            self.local_checker = MlPlatformLocalChecker(api_client=self.api_client)
            self.local_checker.run()
            self.train_do_id = self.local_checker.train_do_id
            self.inference_do_id = self.local_checker.inference_do_id
            self.mount_path = local_mount_path
            Path(self.mount_path).mkdir(parents=True, exist_ok=True)
        if self.api_client.run_mode == RunMode.KUBERNETES:
            self.mlplatform_addr = os.environ["mlplatformAddr"]

            self.mlplatform_client = MlPlatformHttpClient(mlplatform_addr=self.mlplatform_addr)
            self.mlplatform_auth = MlPlatformUserAuth(
                project_id=self.api_client.projectId,
                user_id=self.api_client.userId,
                authorization=self.api_client.Authorization,
                authorization_type=self.api_client.authorizationType,
            )
            self.train_do_id = int(os.environ["doId"]) if "doId" in os.environ else None
            self.inference_do_id = int(os.environ["doId"]) if "doId" in os.environ else None
            self.mount_path = self.api_client.pvc_mount_path
            if "servingMountPath" in os.environ:
                # shutil.rmtree(os.environ["mount_path"])
                self.serving_mount_path = os.environ["servingMountPath"]
                Path(os.path.dirname(self.mount_path)).mkdir(parents=True, exist_ok=True)
                os.symlink(self.serving_mount_path, self.mount_path)
            else:
                Path(self.get_experiment_path()).mkdir(parents=True, exist_ok=True)
                Path(self.get_train_path()).mkdir(parents=True, exist_ok=True)
                if self.inference_id is not None:
                    Path(self.get_inference_path()).mkdir(parents=True, exist_ok=True)

    def download_train_csv(self) -> str:
        return self.hyperdata_api.download_csv(
            do_id=int(self.train_do_id), data_rootpath=self.get_experiment_path(),
        )

    def download_retrain_csv(self) -> str:
        return self.download_train_csv()

    def download_inference_csv(self) -> str:
        return self.hyperdata_api.download_csv(
            do_id=int(self.inference_do_id), data_rootpath=self.get_inference_path(),
        )

    def get_train_id(self) -> int:
        return self.train_id

    def get_inference_id(self) -> int:
        return self.inference_id

    def get_experiment_path(self) -> str:
        experiment_path = os.path.join(self.mount_path, f"experiment-{self.experiment_id}")
        if "servingMountPath" not in os.environ:
            Path(experiment_path).mkdir(parents=True, exist_ok=True)
        return experiment_path

    def get_train_path(self) -> str:
        train_path = os.path.join(self.get_experiment_path(), f"train-{self.train_id}")
        if "servingMountPath" not in os.environ:
            Path(train_path).mkdir(parents=True, exist_ok=True)
        return train_path

    def get_prev_train_path(self) -> List[str]:
        return [
            os.path.join(self.get_experiment_path(), str(train_id))
            for train_id in range(1, self.get_train_id())
        ]

    def get_inference_path(self):
        inference_path = os.path.join(self.get_experiment_path(), f"inference-{self.inference_id}")
        if "servingMountPath" not in os.environ:
            Path(inference_path).mkdir(parents=True, exist_ok=True)
        return inference_path

    def get_inference_csv_path(self):
        return os.path.join(self.get_inference_path(), "inference.csv")

    def insert_model_info(self, model_id: int, model_info: ModelInfoDto) -> ModelInfoDto:
        return self.mlplatform_client.insert_model_info(
            experiment_id=self.experiment_id,
            train_id=self.train_id,
            model_id=model_id,
            model_info=model_info,
            auth=self.mlplatform_auth,
        )



from setuptools import setup, find_packages


# version = {hyperdata_version}.{mlplatform_version}.{union_test_version}.{human_number}.{test_number}
# ex)        8.4.0             .0                   .0.2.1
# mansu: 0
# jooyoun: 1
# yunho: 2
# IF test completed in feature branch, up union_test_version and upload to pypi
# ex. 8.4.0.0.0.0.1, 2, 3 ... -> test completed -> 8.4.0.0.1.0.0
setup(
    name="mlplatform_lib",
    version="8.4.0.0.11.0.1",
    description="mlplatform_lib",
    author="mlplatform_lib",
    author_email="mansu_kim@tmax.co.kr",
    url="",
    download_url="",
    install_requires=[
        "pandas>=1.0.1",
        "requests>=2.25.1",
        "dataclasses_json>=0.5.4",
        "inflection>=0.5.1",
        "dataclasses>=0.6",
    ],
    packages=find_packages(exclude=[]),
    keywords=["mlplatform_lib"],
    python_requires=">=3.6",
    package_data={},
    zip_safe=False,
    classifiers=["Programming Language :: Python :: 3.6"],
)



from typing import List

from mlplatform_lib.mlplatform.mlplatform_http_client import (
    MlPlatformHttpClient,
    MlPlatformUserAuth,
    MlPlatformRequestType,
)
from mlplatform_lib.dataclass.model import ModelAutomlDto, ModelAutomlBestDto
from mlplatform_lib.utils.dataclass_utils import from_dict, to_dict


class AutomlHttpClient(MlPlatformHttpClient):
    def __init__(self, mlplatform_addr):
        super().__init__(mlplatform_addr=mlplatform_addr)

    def get_models(self, experiment_id: int, train_id: int, auth: MlPlatformUserAuth) -> List[ModelAutomlDto]:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            {},
            auth,
            MlPlatformRequestType.READ,
        )
        model_infos = []
        for model_info in res.data:
            model_infos.append(from_dict(ModelAutomlDto, model_info))
        return model_infos

    def insert_model(
        self, experiment_id: int, train_id: int, model: ModelAutomlDto, auth: MlPlatformUserAuth
    ) -> ModelAutomlDto:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            to_dict(model),
            auth,
            MlPlatformRequestType.CREATE,
        )
        return from_dict(ModelAutomlDto, res.data)

    def insert_model_automl_best(
        self,
        experiment_id: int,
        train_id: int,
        model_automl_best: ModelAutomlBestDto,
        auth: MlPlatformUserAuth,
    ) -> ModelAutomlBestDto:
        result = self.send_request(
            "model-automl-bests",
            {"experiments": experiment_id, "trains": train_id},
            {},
            to_dict(model_automl_best),
            auth,
            MlPlatformRequestType.CREATE,
        )
        return from_dict(ModelAutomlBestDto, result.data)



from mlplatform_lib.mlplatform.mlplatform_http_client import (
    MlPlatformHttpClient,
    MlPlatformUserAuth,
    MlPlatformRequestType,
)
from mlplatform_lib.dataclass.model.model_dto import ModelDto
from mlplatform_lib.dataclass.inference.inference_dto import InferenceDto
from mlplatform_lib.utils.dataclass_utils import from_dict, to_dict
from typing import List


class MllabHttpClient(MlPlatformHttpClient):
    def __init__(self, mlplatform_addr):
        super().__init__(mlplatform_addr=mlplatform_addr)

    def get_model_list(self, experiment_id: int, auth: MlPlatformUserAuth) -> List[ModelDto]:
        res = self.send_request(
            "models", {"experiments": experiment_id}, {}, {}, auth, MlPlatformRequestType.READ
        )

        model_infos = []
        print(res.data)
        for model_info in res.data:
            model_infos.append(from_dict(ModelDto, model_info))
        return model_infos

    def get_model_by_id(self, experiment_id: int, train_id: int, auth: MlPlatformUserAuth) -> ModelDto:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            {},
            auth,
            MlPlatformRequestType.READ,
        )

        model_infos = []
        for model_info in res.data:
            model_infos.append(from_dict(ModelDto, model_info))
        return model_infos[0]

    def update_model(
        self, experiment_id: int, train_id: int, dto: ModelDto, auth: MlPlatformUserAuth
    ) -> dict:
        res = self.send_request(
            "models",
            {"experiments": experiment_id, "trains": train_id},
            {},
            to_dict(dto),
            auth,
            MlPlatformRequestType.UPDATE,
        )
        if res.status_code == 200:
            return True
        else:
            return False

    def get_latest_model(self, experiment_id=int, auth=MlPlatformUserAuth) -> ModelDto:
        res = self.send_request(
            "models",
            {"experiments": experiment_id},
            {"latest": "latest"},
            {},
            auth,
            MlPlatformRequestType.READ,
        )
        return from_dict(ModelDto, res.data[0])

    def get_inference_by_id(
        self, experiment_id: int, inference_id: int, auth: MlPlatformUserAuth
    ) -> InferenceDto:
        res = self.send_request(
            "",
            {"experiments": experiment_id, "inferences": inference_id},
            {},
            {},
            auth,
            MlPlatformRequestType.READ,
        )

        return from_dict(InferenceDto, res.data)

    def update_inference(self, experiment_id: int, dto: InferenceDto, auth: MlPlatformUserAuth) -> dict:
        res = self.send_request(
            "inferences",
            {"experiments": experiment_id},
            {},
            to_dict(dto),
            auth,
            MlPlatformRequestType.UPDATE,
        )
        if res.status_code == 200:
            return True
        else:
            return False



from typing import List
from dataclasses import dataclass
from mlplatform_lib.dataclass import DbSourceDataObjectCandidate


@dataclass
class DbSourceDataObjectCandidateList:
    db_source_data_object_candidate: List[DbSourceDataObjectCandidate]



from dataclasses import dataclass, field
from mlplatform_lib.dataclass.experiment.type import ExperimentType
from mlplatform_lib.dataclass.model.type import ModelStatus


@dataclass
class ModelDto:
    id: int = field(init=False, default=0)
    train_id: int = field(init=False)
    name: int
    experiment_type: ExperimentType = field(init=False)
    description: str
    model_path: str
    status: ModelStatus = field(init=False)



from .model_dto import ModelDto
from .model_automl_dto import ModelAutomlDto
from .model_automl_best_dto import ModelAutomlBestDto
from .model_predefinedai_dto import ModelPredefinedaiDto
from .model_info_dto import ModelInfoDto

__all__ = ["ModelDto", "ModelPredefinedaiDto", "ModelInfoDto", "ModelAutomlDto", "ModelAutomlBestDto"]



from .data_object_out_col import DataObjectOutCol
from .data_object import DataObject
from .insert_tuple_object import InsertTupleObject
from .column_of_table_from_default_source import ColumnOfTableFromDefaultSource
from .table_from_default_datasource_info import TableFromDefaultDataSourceInfo
from .table_column_info import TableColumnInfo
from .table_desc_info import TableDescInfo
from .data_object_info_list import DataObjectInfoList
from .db_source_data_object_candidate import DbSourceDataObjectCandidate
from .db_source_data_object_candidate_list import DbSourceDataObjectCandidateList

__all__ = [
    "DataObjectOutCol",
    "DataObject",
    "InsertTupleObject",
    "ColumnOfTableFromDefaultSource",
    "TableFromDefaultDataSourceInfo",
    "TableColumnInfo",
    "TableDescInfo",
    "DataObjectInfoList",
    "DbSourceDataObjectCandidate",
    "DbSourceDataObjectCandidateList"
]



from dataclasses import dataclass

@dataclass
class ExperimentPredefinedaiDto:
    id: int
    authorization_id: int
    experiment_type: str
    name: str
    namespace: str
    pvc_name: str
    pvc_size: str
    created_on: str
    description: str
    do_id: int
    task_type: str